import { CustomResource } from 'aws-cdk-lib';
import { IRole } from 'aws-cdk-lib/aws-iam';
import { IKey, Key } from 'aws-cdk-lib/aws-kms';
import { IFunction } from 'aws-cdk-lib/aws-lambda';
import { ILogGroup } from 'aws-cdk-lib/aws-logs';
import { ISecret } from 'aws-cdk-lib/aws-secretsmanager';
import { Construct } from 'constructs';
import { RedshiftServerlessNamespaceProps } from './redshift-serverless-namespace-props';
import { TrackedConstruct } from '../../../../utils';
/**
 * Create a Redshift Serverless Namespace with the admin credentials stored in Secrets Manager
 *
 * @example
 * const namespace = new dsf.consumption.RedshiftServerlessNamespace(this, 'DefaultServerlessNamespace', {
 *    dbName: 'defaultdb',
 *    name: 'default'
 * });
 */
export declare class RedshiftServerlessNamespace extends TrackedConstruct {
    /**
     * The custom resource that creates the Namespace
     */
    readonly customResource: CustomResource;
    /**
     * The CloudWatch Logs Log Group for the Redshift Serverless creation
     */
    readonly createLogGroup: ILogGroup;
    /**
     * The Lambda Function for the Redshift Serverless creation
     */
    readonly createFunction: IFunction;
    /**
     * The IAM Role for the Redshift Serverless creation
     */
    readonly createRole: IRole;
    /**
     * The CloudWatch Logs Log Group for the creation status check
     */
    readonly statusLogGroup: ILogGroup;
    /**
     * The Lambda Function for the creation status check
     */
    readonly statusFunction: IFunction;
    /**
     * The IAM Role for the creation status check
     */
    readonly statusRole: IRole;
    /**
     * The name of the created namespace
     */
    readonly namespaceName: string;
    /**
     * The created Secrets Manager secret containing the admin credentials
     */
    readonly adminSecret: ISecret;
    /**
     * The KMS Key used to encrypt the admin credentials secret
     */
    readonly adminSecretKey: IKey;
    /**
     * The roles attached to the namespace in the form of `{RoleArn: IRole}`.
     * These roles are used to access other AWS services for ingestion, federated query, and data catalog access.
     * @see https://docs.aws.amazon.com/redshift/latest/mgmt/redshift-iam-authentication-access-control.html
     */
    readonly roles: Record<string, IRole>;
    /**
     * KMS key used by the namespace to encrypt the data
     */
    readonly dataKey: Key;
    /**
     * The name of the database
     */
    readonly dbName: string;
    /**
     * The ARN of the created namespace
     */
    readonly namespaceArn: string;
    /**
     * The ID of the created namespace
     */
    readonly namespaceId: string;
    private readonly removalPolicy;
    /**
     * Used for convenient access to Stack related information such as region and account id.
     */
    private readonly currentStack;
    private namespaceParameters;
    constructor(scope: Construct, id: string, props: RedshiftServerlessNamespaceProps);
}
