import { IRole } from 'aws-cdk-lib/aws-iam';
import { IFunction } from 'aws-cdk-lib/aws-lambda';
import { ILogGroup } from 'aws-cdk-lib/aws-logs';
import { Construct } from 'constructs';
import { RedshiftDataSharingCreateDbProps } from './redshift-data-sharing-createdb-props';
import { RedshiftDataSharingCreateDbFromShareProps } from './redshift-data-sharing-createdbfromshare-props';
import { RedshiftDataSharingGrantProps } from './redshift-data-sharing-grant-props';
import { RedshiftDataSharingGrantedProps } from './redshift-data-sharing-granted-props';
import { RedshiftDataSharingProps } from './redshift-data-sharing-props';
import { RedshiftNewShareProps } from './redshift-new-share-props';
import { BaseRedshiftDataAccess, RedshiftDataAccessTargetProps } from '../base-redshift-data-access';
/**
 * Creates an asynchronous custom resource to manage the data sharing lifecycle for
 * both data producers and data consumers. This also covers both same account and cross account access.
 *
 * @exampleMetadata fixture=vpc-secret
 * @example
 * const redshiftAdminSecret = Secret.fromSecretPartialArn(this, 'RedshiftAdminCredentials', 'arn:aws:secretsmanager:us-east-1:XXXXXXXX:secret:YYYYYYYY');
 *
 * const redshiftVpc = Vpc.fromLookup(this, 'RedshiftVpc', {
 *   vpcId: 'XXXXXXXX',
 * });
 *
 * const dataAccess = new dsf.consumption.RedshiftData(this, 'RedshiftDataAccess', {
 *   workgroupId: 'XXXXXXXXXXXXXXX',
 *   secret: redshiftAdminSecret,
 *   vpc: redshiftVpc,
 *   subnets: redshiftVpc.selectSubnets({
 *     subnetGroupName: 'YYYYYYYY'
 *   }),
 *   createInterfaceVpcEndpoint: true,
 *   executionTimeout: Duration.minutes(10),
 * });
 *
 * const dataShare = new dsf.consumption.RedshiftDataSharing(this, 'RedshiftDataShare', {
 *   redshiftData: dataAccess,
 *   workgroupId: 'XXXXXXXXXXXXXXX',
 *   secret: redshiftAdminSecret,
 *   vpc: redshiftVpc,
 *   subnets: redshiftVpc.selectSubnets({
 *     subnetGroupName: 'YYYYYYYY'
 *   }),
 *   createInterfaceVpcEndpoint: true,
 *   executionTimeout: Duration.minutes(10),
 * });
 *
 *  const share = dataShare.createShare('ProducerShare', 'default', 'example_share', 'public', ['public.customers']);
 *
 *  const grantToConsumer = dataShare.grant('GrantToConsumer', {
 *    dataShareName: 'example_share',
 *    databaseName: 'default',
 *    autoAuthorized: true,
 *    accountId: "<CONSUMER_ACCOUNT_ID>",
 *    dataShareArn: '<DATASHARE_ARN>',
 *  });
 *
 * dataShare.createDatabaseFromShare('ProducerShare', {
 *   consumerNamespaceArn: '',
 *   newDatabaseName: 'db_from_share',
 *   databaseName: 'default',
 *   dataShareName: 'example_share',
 *   dataShareArn: '<DATASHARE_ARN>',
 *   accountId: "<PRODUCER_ACCOUNT_ID>",
 * });
 */
export declare class RedshiftDataSharing extends BaseRedshiftDataAccess {
    /**
     * The IAM Role for the Redshift Data API execution
     */
    readonly executionRole: IRole;
    /**
     * The CloudWatch Log Group for the Redshift Data Sharing submission
     */
    readonly submitLogGroup: ILogGroup;
    /**
     * The Lambda Function for the Redshift Data Sharing submission
     */
    readonly submitFunction: IFunction;
    /**
     * The CloudWatch Log Group for the Redshift Data Sharing status checks
     */
    readonly statusLogGroup: ILogGroup;
    /**
     * The Lambda Function for the Redshift Data Sharing status checks
     */
    readonly statusFunction: IFunction;
    /**
     * The CloudWatch Log Group for the Redshift Data Sharing cleaning up lambda
     */
    readonly cleanUpLogGroup?: ILogGroup;
    /**
     * The Lambda function for the cleaning up lambda
     */
    readonly cleanUpFunction?: IFunction;
    /**
     * The IAM Role for the the cleaning up lambda
     */
    readonly cleanUpRole?: IRole;
    /**
     * Contains normalized details of the target Redshift cluster/workgroup for data access
     */
    readonly dataAccessTargetProps: RedshiftDataAccessTargetProps;
    private readonly serviceToken;
    private readonly redshiftData;
    constructor(scope: Construct, id: string, props: RedshiftDataSharingProps);
    /**
     * Create a new datashare
     * @param id the CDK ID of the resource
     * @param databaseName The name of the database to connect to
     * @param dataShareName The name of the datashare
     * @param schema The schema to add in the datashare
     * @param tables The list of tables that would be included in the datashare. This must follow the format: `<schema>.<tableName>`
     * @returns `RedshiftNewShareProps`
     */
    createShare(id: string, databaseName: string, dataShareName: string, schema: string, tables: string[]): RedshiftNewShareProps;
    /**
     * Create a datashare grant to a namespace if it's in the same account, or to another account
     * @param id the CDK ID of the resource
     * @param props `RedshiftDataSharingGrantProps`
     * @returns `RedshiftDataSharingGrantedProps`
     */
    grant(id: string, props: RedshiftDataSharingGrantProps): RedshiftDataSharingGrantedProps;
    /**
     * Consume datashare by creating a new database pointing to the share.
     * If datashare is coming from a different account, setting `autoAssociate` to true
     * automatically associates the datashare to the cluster before the new database is created.
     * @param id the CDK ID of the resource
     * @param props `RedshiftDataSharingCreateDbProps`
     * @returns `CustomResource`
     */
    createDatabaseFromShare(id: string, props: RedshiftDataSharingCreateDbProps): RedshiftDataSharingCreateDbFromShareProps;
}
