import { CfnCrawler, CfnDatabase, CfnSecurityConfiguration } from 'aws-cdk-lib/aws-glue';
import { AddToPrincipalPolicyResult, IPrincipal, IRole } from 'aws-cdk-lib/aws-iam';
import { IKey } from 'aws-cdk-lib/aws-kms';
import { Construct } from 'constructs';
import { DataCatalogDatabaseProps } from './data-catalog-database-props';
import { TrackedConstruct } from '../../utils';
/**
 * An AWS Glue Data Catalog Database configured with the location and a crawler.
 * @see https://awslabs.github.io/data-solutions-framework-on-aws/docs/constructs/library/Governance/data-catalog-database
 *
 * @example
 * import { Bucket } from 'aws-cdk-lib/aws-s3';
 *
 * new dsf.governance.DataCatalogDatabase(this, 'ExampleDatabase', {
 *    locationBucket: new Bucket(scope, 'LocationBucket'),
 *    locationPrefix: '/databasePath',
 *    name: 'example-db'
 * });
 */
export declare class DataCatalogDatabase extends TrackedConstruct {
    /**
     * The Glue Crawler created when `autoCrawl` is set to `true` (default value). This property can be undefined if `autoCrawl` is set to `false`.
     */
    readonly crawler?: CfnCrawler;
    /**
     * The Glue security configuration used by the Glue Crawler when created.
     */
    readonly crawlerSecurityConfiguration?: CfnSecurityConfiguration;
    /**
     * The IAM Role used by the Glue crawler when created.
     */
    readonly crawlerRole?: IRole;
    /**
     * The Glue Database that's created
     */
    readonly database: CfnDatabase;
    /**
     * The Glue Database name with the randomized suffix to prevent name collisions in the catalog
     */
    readonly databaseName: string;
    /**
     * KMS encryption Key used by the Crawler
     */
    readonly crawlerLogEncryptionKey?: IKey;
    /**
     * Caching constructor properties for internal reuse by constructor methods
     */
    private dataCatalogDatabaseProps;
    constructor(scope: Construct, id: string, props: DataCatalogDatabaseProps);
    /**
     * Grants read access via identity based policy to the principal. This would attach an IAM Policy to the principal allowing read access to the Glue Database and all its Glue Tables.
     * @param principal Principal to attach the Glue Database read access to
     * @returns `AddToPrincipalPolicyResult`
     */
    grantReadOnlyAccess(principal: IPrincipal): AddToPrincipalPolicyResult;
    /**
     * Calculate the table depth level based on the location prefix. This is used by the crawler to determine where the table level files are located.
     * @param locationPrefix `string`
     * @returns `number`
     */
    private calculateDefaultTableLevelDepth;
    /**
     * Based on the parameters passed, it would determine type type of target the crawler would used.
     * @param props `DataCatalogDatabaseProps`
     * @returns `CatalogType`
     */
    private determineCatalogType;
    /**
     * Handle the creation of the crawler with S3 target and its related permissions
     * @param props `DataCatalogDatabaseProps`
     * @param s3Props `S3CrawlerProps`
     * @returns `CfnCrawler`
     */
    private handleS3TypeCrawler;
    /**
     * Handle the creation of the crawler with JDBC target and its related permissions
     * @param props `DataCatalogDatabaseProps`
     * @param jdbcProps `CrawlerProps`
     * @returns `CfnCrawler`
     */
    private handleJDBCTypeCrawler;
}
